/** 
 *******************************************************************************
 * @file    trmosc.c
 * @brief   the application functions of TRMOSC demo for the TOSHIBA
 *          'TMPM372' Device Series 
 * @version 0.0.0.1
 * @date    2014/09/25
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA ELECTRONIC DEVICES & STORAGE CORPORATION 2017 All rights reserved
 *******************************************************************************
 */

/* Includes ------------------------------------------------------------------*/
#include "stdint.h"
#include "stdlib.h"
#include "IOSCadjustment.h"
#include "tmpm372_trmosc.h"
#include "tmrb.h"


/* Private define ------------------------------------------------------------*/
#define  CAPTUREVAL95MHZ		(9895)				/* Caputer value @9.5MHz */
#define  FIXEDPOINT				(100)				/* fixed point 2 */
#define  COARSETRIMSTEP			(180)				/* Coarse trimming step 1.8% * FIXEDPOINT */
#define  FINETRIMSTEP			(30)				/* Fine trimming step   0.3% * FIEXDPOINT */
#define  MARGINADJUST			(50)				/* Adjustment margin    0.5% * FIEXDPOINT */
#define  COARESELIMIT			(0x0000001fUL)		/* Setting limit */
#define  COARESELIMIT_M			(0x00000020UL)		/* Setting limit */
#define  FINELIMIT				(0x00000007UL)		/* Setting limit */
#define  FINELIMIT_M			(0x00000008UL)		/* Setting limit */
#define  TRIMCOARSE_MAX			(0x00000040UL)		/* max value */
#define  TRIMFINE_MAX			(0x0000000FUL)		/* max value */

/* Private variable -----------------------------------------------*/
IOSCADJ_TypeDef IoscAdjust;
TRMOSC_ControlTypeDef AdjustData;
int32_t  difference;

/* Private function prototypes -----------------------------------------------*/
void MeasureWave(IOSCADJ_TypeDef *Measure);
int32_t CalcDiff(IOSCADJ_TypeDef *Diff);
int32_t CalcAdjVal(IOSCADJ_TypeDef *Adjust);
int32_t GetAdjustmentValue(void);

/**
  * @brief  Adjustment of internal OSC
  * @param  None
  * @retval None
  */
int32_t IOSCAdjustment(void)
{
	int32_t retval;

	MeasureWave(&IoscAdjust);											/* Measurement Waveform */
	retval = CalcDiff(&IoscAdjust);										/* Calculation of difference */
	if(retval == TRMOSC_ERROR) {
		retval = CalcAdjVal(&IoscAdjust);								/* Calculation of adjustment value */
		if(retval == TRMOSC_RUN) {
			AdjustData.TrimmingControl     = TRMOSC_CONTROL_ENABLE;
			AdjustData.CoarseTrimmingValue = IoscAdjust.CoarseTrim;
			AdjustData.FineTrimmingValue   = IoscAdjust.FineTrim;
			SetTrmOsc(&AdjustData);										/* Set TRMOSC register */
		}
		else {															/* TRMOSC_ERROR */
			/* do nothing */
		}
	}
	else {																/* TRMOSC_DONE  */
		/* do nothing */
	}
	return (retval);
}

/**
  * @brief  Measurement Waveform
  * @param  TRMOSC adjustment Structure
  * @retval None
  */
void MeasureWave(IOSCADJ_TypeDef *Measure)
{
	StartCapture();
	Measure->MeasureVal = GetCaptureData();
	StopCapture();
}

/**
  * @brief  Calculation of difference
  * @param  TRMOSC adjustment Structure
  * @retval TRMOSC_DONE, TRMOSC_ERROR
  */
int32_t CalcDiff(IOSCADJ_TypeDef *Diff)
{
	uint32_t absdiff;
	int32_t retval;

	Diff->MeasureDiff = ((CAPTUREVAL95MHZ * (100*FIXEDPOINT)) / Diff->MeasureVal)
				 - (100*FIXEDPOINT);
/**/
	absdiff = abs(Diff->MeasureDiff);
	if(absdiff <= MARGINADJUST) {
		retval = TRMOSC_DONE;
	}
	else {
		retval = TRMOSC_ERROR;
	}

	return (retval);
}

/**
  * @brief  Calculation of adjustment value
  * @param  TRMOSC adjustment Structure
  * @retval TRMOSC_RUN, TRMOSC_ERROR
  */
int32_t CalcAdjVal(IOSCADJ_TypeDef *Adjust)
{
	int32_t retval;
	uint32_t coarse, fine;

	difference = Adjust->MeasureDiff + GetAdjustmentValue();
	coarse =  ((uint32_t)abs(difference) / COARSETRIMSTEP);
	fine   = (((uint32_t)abs(difference) - (coarse * COARSETRIMSTEP)) / FINETRIMSTEP);

	if(difference < 0){
		if((coarse <= COARESELIMIT_M) && (fine <= FINELIMIT_M)) {
			Adjust->CoarseTrim = (int32_t)-coarse;
		    Adjust->FineTrim   = (int32_t)-fine;
			retval = TRMOSC_RUN;
		}
		else {
			retval = TRMOSC_ERROR;
		}
	}
	else {
		if((coarse <= COARESELIMIT) && (fine <= FINELIMIT)) {
			Adjust->CoarseTrim = (int32_t)coarse;
		    Adjust->FineTrim   = (int32_t)fine;
			retval = TRMOSC_RUN;
		}
		else {
			retval = TRMOSC_ERROR;
		}
	}

	return (retval);
}

/**
  * @brief  Get Adjustment value
  * @param  None
  * @retval Adjustment value
  */
int32_t GetAdjustmentValue(void)
{
	uint32_t trimvalueC;
	uint32_t trimvalueF;
	int32_t adjvalue;

	if(GetTrimmingControlValue() == TRMOSC_CONTROL_DISABLE) {
		trimvalueC = GetInitCoarseTrimValue();
		trimvalueF = GetInitFineTrimValue();
	}
	else {
		trimvalueC = GetCoarseTrimValueSet();
		trimvalueF = GetFineTrimValueSet();
	}
/**/
	if(trimvalueC > COARESELIMIT) {
		trimvalueC = TRIMCOARSE_MAX - trimvalueC;
		adjvalue = -COARSETRIMSTEP * (int32_t)trimvalueC;
	}
	else {
		adjvalue = COARSETRIMSTEP * (int32_t)trimvalueC;
	}
/**/
	if(trimvalueF > FINELIMIT) {
		trimvalueF = TRIMFINE_MAX - trimvalueF;
		adjvalue += -FINETRIMSTEP * (int32_t)trimvalueF;
	}
	else {
		adjvalue += FINETRIMSTEP * (int32_t)trimvalueF;
	}

	return (adjvalue);
}

/**
  * @brief  Initial TRMOSC for debug
  * @param  None
  * @retval None
  */
void InitialTRMOSC(void)
{
	TRMOSC_ControlTypeDef InitData;

	InitData.TrimmingControl     = TRMOSC_CONTROL_DISABLE;
	InitData.CoarseTrimmingValue = 0;
	InitData.FineTrimmingValue   = 0;
	SetTrmOsc(&InitData);
}
